package fishrungames.bashgid;


import java.util.ArrayList;

import fishrungames.bashgid.core.BookManager;
import fishrungames.networkutils.ImageManager;
import fishrungames.networkutils.UniversalActionWithCallback;
import fishrungames.networkutils.interfaces.RemoveCallbackInterface;
import fishrungames.networkutils.interfaces.UpdateAndFinishCallbackInterface;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.ImageView;

import android.widget.TextView;


public class BooksFragment extends Fragment implements UpdateAndFinishCallbackInterface
{

	ListAdapter listAdapter;

	GridView gridView;

	ArrayList<BookManager.BookRecord> bookRecordArr;

	RemoveCallbackInterface removeCallback = null;
	Object removeCallbackSynchronizer = new Object();



	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {


		bookRecordArr = BookManager.getInstance().getBooks();

		removeCallback = UniversalActionWithCallback.PerformActionIfNotPerforming(BookManager.getInstance(), null, this, "BookManager");


		View rootView = inflater.inflate(R.layout.fragment_books_page, container, false);

		gridView = (GridView) rootView.findViewById(R.id.gridView);

		listAdapter = new ListAdapter(getActivity());

		gridView.setAdapter(listAdapter);

		gridView.setOnItemClickListener(new OnItemClickListener()
		{

			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id)
			{
				MainActivity.getInstance().OpenBookRecordScreen(bookRecordArr.get(position), MainActivity.TAG_FROM_BOOKSFRAGMENT_TO_BOOKRECORDFRAGMENT);
			}

		});

		RefreshBookList();

		return rootView;
	}

	@Override
	public void onDestroyView()
	{
		synchronized (removeCallbackSynchronizer)
		{
			if (removeCallback != null)
			{
				removeCallback.RemoveCallback();
				removeCallback = null;
			}
		}
		super.onDestroyView();
	}

	public void RefreshBookList()
	{
		bookRecordArr = BookManager.getInstance().getBooks();

		listAdapter.notifyDataSetChanged();
	}

	@Override
	public void OnFinished()
	{
		synchronized (removeCallbackSynchronizer)
		{
			removeCallback = null;
		}

	}

	@Override
	public void OnUpdated(Bundle result)
	{
		getActivity().runOnUiThread(new Runnable() {

			@Override
			public void run()
			{
				RefreshBookList();
			}

		});
	}

	public class ListAdapter extends BaseAdapter {
		private Context mContext;

		public ListAdapter(Context c) {
			mContext = c;
		}

		public int getCount() {
			return bookRecordArr.size();
		}

		public Object getItem(int position) {
			return null;
		}

		public long getItemId(int position) {
			return 0;
		}

		// create a new ImageView for each item referenced by the Adapter
		@SuppressLint("InflateParams")
		public View getView(int position, View convertView, ViewGroup parent) {


			View view;
			if (convertView == null) {  // if it's not recycled, initialize some attributes
				LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				view = inflater.inflate(R.layout.books_page_list_element, null);


			} else {
				view = convertView;
			}

			ImageView imageView = (ImageView)view.findViewById(R.id.imageView);
			TextView nameTextView = (TextView) view.findViewById(R.id.nameTextView);
			TextView authorTextView = (TextView) view.findViewById(R.id.authorTextView);

			if (bookRecordArr.get(position).imageUrl.length() == 0)
			{
				imageView.setImageResource(R.drawable.no_picture);
			}
			else
			{
				ImageManager.getInstance().ApplyImageToImageView(imageView, bookRecordArr.get(position).imageUrl);
			}

			nameTextView.setText(bookRecordArr.get(position).name);

			authorTextView.setText(bookRecordArr.get(position).author);

			return view;
		}

	}
}
