package fishrungames.bashgid;



import java.util.ArrayList;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v4.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;


import org.osmdroid.DefaultResourceProxyImpl;
import org.osmdroid.ResourceProxy;
import org.osmdroid.api.IMapController;
import org.osmdroid.api.IMapView;
import org.osmdroid.util.GeoPoint;

import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.ItemizedOverlay;

import org.osmdroid.views.overlay.OverlayItem;
import org.osmdroid.views.overlay.OverlayItem.HotspotPlace;

import fishrungames.bashgid.core.ArticleManager.ArticleRecordData;
import fishrungames.bashgid.core.DatabaseGetJournalTask;
import fishrungames.bashgid.core.JournalManager;
import fishrungames.bashgid.core.LocationHolder;
import fishrungames.bashgid.core.JournalManager.JournalSemiFullData;
import fishrungames.networkutils.ImageManager;


public class OsmMapHolderFragment extends Fragment
{

	public static class CustomItemizedOverlay extends ItemizedOverlay<OverlayItem>{

		private ArrayList<OverlayItem> mOverlayItems = new ArrayList<OverlayItem>();
		private ArrayList<String> mArticles = new ArrayList<String>();
		String journalName;

		public CustomItemizedOverlay(Drawable pDefaultMarker, ResourceProxy pResourceProxy, String journalName) {
			super(pDefaultMarker, pResourceProxy);
			this.journalName = journalName;
		}

		@Override
		protected OverlayItem createItem(int pos) {
			return mOverlayItems.get(pos);
		}

		@Override
		public int size() {
			if(mOverlayItems!=null) return mOverlayItems.size();
			else return 0;
		}

		public void addOverlayItem(GeoPoint geoPoint, String articleName, String title, String imageUrl){

			OverlayItem item = new OverlayItem(title, imageUrl, geoPoint);
			item.setMarker(MainActivity.getInstance().getResources().getDrawable(R.drawable.marker_default));
			item.setMarkerHotspot(HotspotPlace.CENTER);

			mOverlayItems.add(item);
			mArticles.add(articleName);

			populateNow();
		}

		public void populateNow(){
			populate();
		}

		@Override
		public boolean onSnapToItem(int arg0, int arg1, Point arg2, IMapView arg3)
		{
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		protected boolean onTap(final int index) {
			OverlayItem item = mOverlayItems.get(index);
			AlertDialog.Builder dialog = new AlertDialog.Builder(MainActivity.getInstance());
			dialog.setTitle(item.getTitle());
			//dialog.setMessage(item.getSnippet());

			ImageView image = new ImageView(MainActivity.getInstance());
			ImageManager.getInstance().ApplyImageToImageView(image, item.getSnippet());
			//image.setImageResource(R.drawable.YOUR_IMAGE_ID);

			dialog.setView(image);


			dialog.setPositiveButton("Open", new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					MainActivity.getInstance().SetLastMapFilterName(journalName);
					MainActivity.getInstance().OpenArticleScreen(mArticles.get(index), MainActivity.TAG_FROM_MAPFRAGMENT_TO_ARTICLEFRAGMENT);
				}
			});

			dialog.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int id) {
					dialog.cancel();
				}
			});




			dialog.show();


			return true;
		}
	}


	public static class NoInteractionOverlay extends ItemizedOverlay<OverlayItem>{
		private ArrayList<OverlayItem> mOverlayItems = new ArrayList<OverlayItem>();

		public NoInteractionOverlay(Drawable pDefaultMarker, ResourceProxy pResourceProxy) {
			super(pDefaultMarker, pResourceProxy);
		}

		@Override
		protected OverlayItem createItem(int pos) {
			return mOverlayItems.get(pos);
		}

		@Override
		public int size() {
			if(mOverlayItems!=null) return mOverlayItems.size();
			else return 0;
		}

		public void addOverlayItem(GeoPoint geoPoint){

			OverlayItem item = new OverlayItem("", "", geoPoint);
			item.setMarker(MainActivity.getInstance().getResources().getDrawable(R.drawable.marker_default));
			item.setMarkerHotspot(HotspotPlace.CENTER);

			mOverlayItems.add(item);

			populateNow();
		}

		public void populateNow(){
			populate();
		}

		@Override
		public boolean onSnapToItem(int arg0, int arg1, Point arg2, IMapView arg3)
		{
			return false;
		}
	}

	CustomItemizedOverlay itemizedOverlay;

	String journalName;

	JournalSemiFullData journal;


	public OsmMapHolderFragment()
	{
		journal = null;
		journalName = "";
	}


	public OsmMapHolderFragment(String journalName)
	{
		this.journalName = journalName;
		RestoreJournal();
	}

	@Override
	public void onSaveInstanceState(final Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putString("journalName", journalName);
	}


	MapView mapView;
	TextView textView;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		if (savedInstanceState != null) {
			journalName = savedInstanceState.getString("journalName");
			RestoreJournal();
		}

		View rootView = inflater.inflate(R.layout.fragment_osm_map_page, container, false);

		textView = (TextView) rootView.findViewById(R.id.textView);

		textView.setText("Loading map...");

		mapView = (MapView) rootView.findViewById(R.id.mapview);

		mapView.setMaxZoomLevel(20);
		mapView.setMinZoomLevel(4);

		mapView.setBuiltInZoomControls(true);
		mapView.setMultiTouchControls(true);


		IMapController mapController = mapView.getController();		

		GeoPoint center = new GeoPoint(LocationHolder.CONST_UFA_LATITUDE, LocationHolder.CONST_UFA_LONGITUDE);

		mapController.setZoom(10);
		mapController.animateTo(center);


		RefreshView();

		return rootView;
	}


	void RestoreJournal()
	{
		journal = JournalManager.CreateEmptyJournal();

		DatabaseGetJournalTask task = new DatabaseGetJournalTask();

		task.mHandler = MainActivity.getInstance().mHandler;

		Bundle bundle = new Bundle();

		bundle.putString("journalName", journalName);

		task.execute(bundle);


	}

	public void SetJournal(JournalSemiFullData journal)
	{
		textView.setText("Map is loaded");

		this.journal = journal;

		RefreshView();

	}

	public void RefreshView()
	{
		if (journal != null)
		{   
			ResourceProxy resourceProxy = new DefaultResourceProxyImpl(getActivity());
			itemizedOverlay = new CustomItemizedOverlay(getResources().getDrawable(R.drawable.russia_icon), resourceProxy, journalName);

			for (int i = 0; i < journal.articleArr.size(); i++)
			{
				ArticleRecordData article = journal.articleArr.get(i);

				if (article.geoLat != 0 || article.geoLon != 0)
				{
					GeoPoint markerPoint = new GeoPoint(article.geoLat, article.geoLon);

					itemizedOverlay.addOverlayItem(markerPoint, article.name, article.title, journal.articleArr.get(i).previewImageUrl);

				}
			}

			mapView.getOverlays().clear();
			mapView.getOverlays().add(itemizedOverlay);

			mapView.invalidate();

		}
	}

}
